/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package com.amazonaws.services.batch.model;

import java.io.Serializable;
import javax.annotation.Generated;
import com.amazonaws.protocol.StructuredPojo;
import com.amazonaws.protocol.ProtocolMarshaller;

/**
 * <p>
 * The retry strategy associated with a job.
 * </p>
 * 
 * @see <a href="http://docs.aws.amazon.com/goto/WebAPI/batch-2016-08-10/RetryStrategy" target="_top">AWS API
 *      Documentation</a>
 */
@Generated("com.amazonaws:aws-java-sdk-code-generator")
public class RetryStrategy implements Serializable, Cloneable, StructuredPojo {

    /**
     * <p>
     * The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts.
     * If the value of <code>attempts</code> is greater than one, the job is retried if it fails until it has moved to
     * <code>RUNNABLE</code> that many times.
     * </p>
     */
    private Integer attempts;

    /**
     * <p>
     * The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts.
     * If the value of <code>attempts</code> is greater than one, the job is retried if it fails until it has moved to
     * <code>RUNNABLE</code> that many times.
     * </p>
     * 
     * @param attempts
     *        The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10
     *        attempts. If the value of <code>attempts</code> is greater than one, the job is retried if it fails until
     *        it has moved to <code>RUNNABLE</code> that many times.
     */

    public void setAttempts(Integer attempts) {
        this.attempts = attempts;
    }

    /**
     * <p>
     * The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts.
     * If the value of <code>attempts</code> is greater than one, the job is retried if it fails until it has moved to
     * <code>RUNNABLE</code> that many times.
     * </p>
     * 
     * @return The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10
     *         attempts. If the value of <code>attempts</code> is greater than one, the job is retried if it fails until
     *         it has moved to <code>RUNNABLE</code> that many times.
     */

    public Integer getAttempts() {
        return this.attempts;
    }

    /**
     * <p>
     * The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts.
     * If the value of <code>attempts</code> is greater than one, the job is retried if it fails until it has moved to
     * <code>RUNNABLE</code> that many times.
     * </p>
     * 
     * @param attempts
     *        The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10
     *        attempts. If the value of <code>attempts</code> is greater than one, the job is retried if it fails until
     *        it has moved to <code>RUNNABLE</code> that many times.
     * @return Returns a reference to this object so that method calls can be chained together.
     */

    public RetryStrategy withAttempts(Integer attempts) {
        setAttempts(attempts);
        return this;
    }

    /**
     * Returns a string representation of this object; useful for testing and debugging.
     *
     * @return A string representation of this object.
     *
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        if (getAttempts() != null)
            sb.append("Attempts: ").append(getAttempts());
        sb.append("}");
        return sb.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;

        if (obj instanceof RetryStrategy == false)
            return false;
        RetryStrategy other = (RetryStrategy) obj;
        if (other.getAttempts() == null ^ this.getAttempts() == null)
            return false;
        if (other.getAttempts() != null && other.getAttempts().equals(this.getAttempts()) == false)
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int hashCode = 1;

        hashCode = prime * hashCode + ((getAttempts() == null) ? 0 : getAttempts().hashCode());
        return hashCode;
    }

    @Override
    public RetryStrategy clone() {
        try {
            return (RetryStrategy) super.clone();
        } catch (CloneNotSupportedException e) {
            throw new IllegalStateException("Got a CloneNotSupportedException from Object.clone() " + "even though we're Cloneable!", e);
        }
    }

    @com.amazonaws.annotation.SdkInternalApi
    @Override
    public void marshall(ProtocolMarshaller protocolMarshaller) {
        com.amazonaws.services.batch.model.transform.RetryStrategyMarshaller.getInstance().marshall(this, protocolMarshaller);
    }
}
